# http://scikit-learn.org/stable/auto_examples/linear_model/plot_sgd_comparison.html#sphx-glr-auto-examples-linear-model-plot-sgd-comparison-py

import time

import numpy as np

import matplotlib.pyplot as plt

from sklearn.base import clone

from sklearn.datasets import load_digits

from sklearn.model_selection import train_test_split

from define import *


def compare(classifiers, holdout=HOLDOUT, rng=np.random.RandomState(1), rounds=ROUNDS, dataset=load_digits()):
    X, y = dataset.data, dataset.target
    result = []

    xx = 1. - np.array(holdout)

    for name, clf in classifiers:
        holdout_errors = []
        for i in holdout:
            current_holdout_errors = []
            total_time = 0

            for r in range(rounds):
                current_clf = clone(clf)
                X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=i, random_state=rng)

                start = time.time()
                current_clf.fit(X_train, y_train)
                total_time += time.time() - start

                current_holdout_errors.append(1 - current_clf.score(X_test, y_test))

            holdout_errors.append(np.mean(current_holdout_errors))

            print("Training {} with {:.1f}% training data took {:.03f}s. Errors: {:.3f}".format(name, round((1-i)*100, 0), total_time, np.mean(current_holdout_errors)))

        result.append((name, holdout_errors))
        plt.plot(xx, holdout_errors, label=name)

    plt.gca().set_xlim(left=0, right=1)
    plt.gca().set_ylim(bottom=0)
    plt.legend(loc="upper right")
    plt.xlabel("Proportion train")
    plt.ylabel("Test Error Rate")
    plt.show()

    return result

